/*
 *  linux/arch/arm/mach-ep93xx/time.c
 *
 *  Copyright (C) 2000-2001 Deep Blue Solutions
 *
 * (c) Copyright 2001 LynuxWorks, Inc., San Jose, CA.  All rights reserved.
 *  Copyright (C) 2002-2003 Cirrus Logic, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#include <linux/timex.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/sched.h>

#include <asm/hardware.h>
#include <asm/irq.h>
#include <asm/io.h>

#include <asm/mach/time.h>

/*
 * gettimeoffset() returns time since last timer tick, in usecs.
 *
 * 'LATCH' is hwclock ticks (see CLOCK_TICK_RATE in timex.h) per jiffy.
 * 'tick' is usecs per jiffy.
 */
static unsigned long ep93xx_gettimeoffset(void)
{
	unsigned long hwticks;
	hwticks = LATCH - (inl(TIMER1VALUE) & 0xffff);
	return (hwticks * (tick_nsec / 1000)) / LATCH;
}

/*
 * IRQ handler for the timer
 */
static irqreturn_t
ep93xx_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
#if defined(CLOCK_CORRECT) && CLOCK_CORRECT
	static unsigned long clock_correct = CLOCK_CORRECT;
	if (!--clock_correct) clock_correct = CLOCK_CORRECT + 1;
	else
#endif
	outl(1, TIMER1CLEAR);
	timer_tick(regs);
	return IRQ_HANDLED;
}

static struct irqaction ep93xx_timer_irq = {
	.name		= "timer tick",
	.flags		= SA_INTERRUPT,
	.handler	= ep93xx_timer_interrupt
};

void __init ep93xx_init_time(void)
{
	struct timespec tv;

	outl(0, TIMER1CONTROL);
	outl(LATCH - 1, TIMER1LOAD);
	outl(0xc8, TIMER1CONTROL);

	setup_irq(IRQ_TIMER1, &ep93xx_timer_irq);

	tv.tv_nsec = 0;
	tv.tv_sec = inl(RTCDR);
	do_settimeofday(&tv);
}

struct sys_timer ep93xx_timer = {
	.init		= ep93xx_init_time,
	.offset		= ep93xx_gettimeoffset,
};
